MADHUSUDANA_MASA = 0
TRIVIKRAMA_MASA = 1
VAMANA_MASA = 2
SRIDHARA_MASA = 3
HRSIKESA_MASA = 4
PADMANABHA_MASA = 5
DAMODARA_MASA = 6
KESAVA_MASA = 7
NARAYANA_MASA = 8
MADHAVA_MASA = 9
GOVINDA_MASA = 10
VISNU_MASA = 11
ADHIKA_MASA = 12
