PRIO_MAHADVADASI = 10
PRIO_EKADASI = 20
PRIO_EKADASI_PARANA = 90
PRIO_FESTIVALS_0 = 100
PRIO_FESTIVALS_1 = 200
PRIO_FESTIVALS_2 = 300
PRIO_FESTIVALS_3 = 400
PRIO_FESTIVALS_4 = 500
PRIO_FESTIVALS_5 = 600
PRIO_FESTIVALS_6 = 700
PRIO_FASTING = 900
PRIO_SANKRANTI = 920
PRIO_MASA_CHANGE = 940
PRIO_DST_CHANGE = 950
PRIO_KSAYA = 965
PRIO_CM_CONT = 971
PRIO_CM_DAY = 972
PRIO_CM_DAYNOTE = 973
PRIO_ARUN = 975
PRIO_SUN  = 980
PRIO_MOON = 990
PRIO_ASTRO = 1000
