
class GCHorizontalCoords:
    def __init__(self):
    	self.azimut = 0.0
    	self.elevation = 0.0


class GCEquatorialCoords:
    def __init__(self):
    	self.rightAscension = 0.0
    	self.declination = 0.0

class GCEclipticalCoords:
    def __init__(self):
    	self.latitude = 0.0
    	self.longitude = 0.0
    	self.distance = 0.0
