DISP_ALWAYS = -1
CAL_ARUN_TIME  = 1
CAL_ARUN_TITHI = 0
CAL_SUN_RISE = 2
CAL_SUN_NOON = 34
CAL_SUN_SET = 3
CAL_MOON_RISE = 4
CAL_MOON_SET = 5
CAL_KSAYA = 7
CAL_VRDDHI = 8
CAL_SUN_LONG = 9
CAL_MOON_LONG = 10
CAL_AYANAMSHA = 11
CAL_JULIAN = 12
CATURMASYA_PURNIMA = 13
CATURMASYA_PRATIPAT = 14
CATURMASYA_EKADASI = 15
CAL_SANKRANTI = 16
CAL_EKADASI_PARANA = 17
CAL_MASA_CHANGE = 21
CAL_FEST_0 = 22
CAL_FEST_1 = 23
CAL_FEST_2 = 24
CAL_FEST_3 = 25
CAL_FEST_4 = 26
CAL_FEST_5 = 27
CAL_FEST_6 = 28
CAL_DST_CHANGE = 35
GENERAL_FIRST_DOW = 40
COREEVENTS_SUN = 52
COREEVENTS_TITHI = 53
COREEVENTS_NAKSATRA = 54
COREEVENTS_SANKRANTI = 55
COREEVENTS_CONJUNCTION = 56
COREEVENTS_RAHUKALAM = 57
COREEVENTS_YAMAGHANTI = 58
COREEVENTS_GULIKALAM = 59
COREEVENTS_MOON = 60
COREEVENTS_MOONRASI = 61
COREEVENTS_ASCENDENT = 62
COREEVENTS_SORT = 63
COREEVENTS_ABHIJIT_MUHURTA = 64
COREEVENTS_YOGA = 65
