CMASYA_SYSTEM_PURNIMA = 13
CMASYA_SYSTEM_PRATIPAT = 14
CMASYA_SYSTEM_EKADASI = 15
CMASYA_SYSTEM_MASK = 0xff

CMASYA_MONTH_1 = 0x100
CMASYA_MONTH_2 = 0x200
CMASYA_MONTH_3 = 0x300
CMASYA_MONTH_4 = 0x400
CMASYA_MONTH_MASK = 0xf00

CMASYA_DAY_FIRST = 0x1000
CMASYA_DAY_LAST  = 0x2000
CMASYA_DAY_MASK  = 0xf000
